// Interactive cropping by dragging a rectangle.

PImage img;

// Don't call this variable "dragging", since we want the
// rectangle to persist even after the mouse is released.
boolean have_rect;

int downx;
int downy;
int curx;
int cury;

void setup()
{
  img = loadImage( "otis.jpg" );
  // Make extra room for the cropped version.
  size( 2*img.width, img.height );
  have_rect = false;
}

// Helper function do the actual cropping.
PImage crop()
{
  if( have_rect ) {
    int x = min( downx, curx );
    int y = min( downy, cury );
    int w = abs( downx - curx );
    int h = abs( downy - cury );
    
    // This is the only new part of the sketch -- extract out
    // a rectangle from the current image.
    return img.get( x, y, w, h );
  } else {
    // If there isn't an actual current rectangle, it doesn't
    // make sure to return any specific PImage.  So take advantage
    // of null!  Return the null value to indicate that the
    // operation failed.
    return null;
  }
}

void draw()
{
  background( 0 );
  image( img, 0, 0 );
  if( have_rect ) {
    noFill();
    stroke( #ffff00 );
    strokeWeight( 2 );
    rect( downx, downy, curx - downx, cury - downy );
    image( crop(), img.width, 0 );
  }
}

// Start a mouse interaction.  Set the start and end of the
// rectangle to the mouse location.
void mousePressed()
{
  have_rect = true;
  downx = mouseX;
  downy = mouseY;
  curx = mouseX;
  cury = mouseY;
}

void mouseDragged()
{
  curx = mouseX;
  cury = mouseY;
}

// If you drag out a degenerate rectangle, assume the user
// wants to not have a rectangle on the screen at all.  This
// is a natural interaction on screen -- you can click once
// without moving the mouse to "dismiss" the current crop
// rectangle.
void mouseReleased()
{
  if( (downx == curx) || (downy == cury) ) {
    have_rect = false;
  }
}
