// A demonstration of what a blurring algorithm looks like
// under the hood (spoiler: it's a mess).  This isn't a
// very good blurring algorithm, but it shows off the basics.

PImage img;

// This function behaves just like reading a single
// pixel from the image, but it explicitly checks 
// if the coordinates are in-bounds and returns black
// if they're not.  That way you can pretend that a
// finite image extends out to infinity in every direction
// with black pixels.
color sampleImage( PImage im, int x, int y )
{
  if( x < 0 ) {
    return color( 0 );
  }
  if( x >= im.width ) {
    return color( 0 );
  }
  if( y < 0 ) {
    return color( 0 );
  }
  if( y >= im.height ) {
    return color( 0 );
  }
  
  return im.pixels[y*im.width+x];
}

// The "kernel" for blurring: weights to apply to the
// nine pixels in a 3x3 block around a central pixel.
int[] weights = { 
	1, 2, 1,
	2, 4, 2,
	1, 2, 1 };

PImage blurImage( PImage im )
{
  PImage res = createImage( im.width, im.height, RGB );
  
  im.loadPixels();
  res.loadPixels();
  
  // Yikes, four levels of nested loops!
  for( int y = 0; y < img.height; ++y ) {
    for( int x = 0; x < img.width; ++x ) {
      int r = 0;
      int g = 0;
      int b = 0;
      for( int i = -1; i <= 1; ++i ) {
        for( int j = -1; j <= 1; ++j ) {
          color c = sampleImage( im, x+j, y+i );
          int idx = (i+1)*3+(j+1);
          r += red( c ) * weights[idx];
          g += green( c ) * weights[idx];
          b += blue( c ) * weights[idx];
        }
      }
      
      res.pixels[ y*res.width + x ] = 
        color( r/16.0, g/16.0, b/16.0 );
    }
  }
  res.updatePixels();
  return res;
}

void setup()
{
  img = loadImage( "otis.jpg" );
  size( img.width, img.height );
  frameRate( 5 );
}

void draw()
{
  image( img, 0, 0 );
  img = blurImage( img );
}
